package types

import (
	"encoding/json"
	"fmt"
	"io"
	"time"
)

// Sensor ...
type Sensor struct {
	SensorID          string      `json:"sensor_id" xml:"sensor_id"`
	SensorName        string      `json:"sensor_name" xml:"sensor_name"`
	SensorLocation    string      `json:"sensor_location" xml:"sensor_location"`
	WireID            *string     `json:"wire_id" xml:"wire_id"`
	GPIONumber        *GPIO       `json:"gpio_number" xml:"gpio_number"`
	SensorModel       SensorModel `json:"sensor_model" xml:"sensor_model"`
	SensorEnabled     bool        `json:"sensor_enabled" xml:"sensor_enabled"`
	SensorLastContact time.Time   `json:"sensor_last_contact" xml:"sensor_last_contact"`
	DeviceID          string      `json:"device_id" xml:"device_id"`
	CreationDate      time.Time   `json:"creation_date" xml:"creation_date"`
}

// JSONDecoder decodes a json into a sensor
func (s *Sensor) JSONDecoder(r io.Reader) error {
	decoder := json.NewDecoder(r)
	err := decoder.Decode(s)
	if err != nil {
		return fmt.Errorf("Can not decode sensor from json: %v", err)
	}
	return nil
}

func (s *Sensor) JSONEncoder(w io.Writer) error {
	encoder := json.NewEncoder(w)
	encoder.SetIndent("", " ")
	err := encoder.Encode(s)
	if err != nil {
		return fmt.Errorf("Can not encode sensor to json: %v", err)
	}
	return nil
}

func (s *Sensor) Name() string {
	if s.SensorName != "" {
		return s.SensorName
	} else if s.WireID != nil {
		return *s.WireID
	}
	return s.SensorID
}
